/**
 * Copyright (c) 2014 - 2018, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/** @file
 *
 * @defgroup ble_sdk_uart_over_ble_main main.c
 * @{
 * @ingroup  ble_sdk_app_nus_eval
 * @brief    UART over BLE application main file.
 *
 * This file contains the source code for a sample application that uses the Nordic UART service.
 * This application uses the @ref srvlib_conn_params module.
 */


#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "ble_hci.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_params.h"
#include "nrf_sdh.h"
#include "nrf_sdh_soc.h"
#include "nrf_sdh_ble.h"
#include "nrf_ble_gatt.h"
#include "nrf_ble_qwr.h"
#include "app_timer.h"
#include "ble_nus.h"
#include "app_uart.h"
#include "app_util_platform.h"
#include "bsp_btn_ble.h"
#include "nrf_pwr_mgmt.h"
#include "nrf_delay.h"
#include "nrf_drv_saadc.h"
#include "nrf_uart.h"


#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"


//ԼĿ
#include "my_motor.h"
#include "my_oled.h"
#include "my_dust.h"

//dfuؿ
#include "ble_srv_common.h"
#include "nrf_dfu_ble_svci_bond_sharing.h"
#include "nrf_svci_async_function.h"
#include "nrf_svci_async_handler.h"
#include "ble_dfu.h"
#include "peer_manager.h"
#include "fds.h"
#include "ble_conn_state.h"
#include "ble.h"
#include "nrf_power.h"

//dfuغ궨
 #define SEC_PARAM_BOND 1
/**< Perform bonding. */
#define SEC_PARAM_MITM 0
/**< Man In The Middle protection not required. */
#define SEC_PARAM_LESC 0
/**< LE Secure Connections not enabled. */
#define SEC_PARAM_KEYPRESS 0
/**< Keypress notifications not enabled. */
#define SEC_PARAM_IO_CAPABILITIES BLE_GAP_IO_CAPS_NONE
/**< No I/O capabilities. */
#define SEC_PARAM_OOB 0
/**< Out Of Band data not available. */
#define SEC_PARAM_MIN_KEY_SIZE 7
/**< Minimum encryption key size. */
#define SEC_PARAM_MAX_KEY_SIZE 16
/**< Maximum encryption key size. */

#define APP_BLE_CONN_CFG_TAG            1                                           /**< A tag identifying the SoftDevice BLE configuration. */

#define DEVICE_NAME                     "Nordic_UART"                               /**< Name of device. Will be included in the advertising data. */
#define NUS_SERVICE_UUID_TYPE           BLE_UUID_TYPE_VENDOR_BEGIN                  /**< UUID type for the Nordic UART Service (vendor specific). */

#define APP_BLE_OBSERVER_PRIO           3                                           /**< Application's BLE observer priority. You shouldn't need to modify this value. */

#define APP_ADV_INTERVAL                3200                                          /**< The advertising interval (in units of 0.625 ms. This value corresponds to 40 ms). */

#define APP_ADV_DURATION                0                                       /**< The advertising duration (180 seconds) in units of 10 milliseconds. */

#define MIN_CONN_INTERVAL               MSEC_TO_UNITS(20, UNIT_1_25_MS)             /**< Minimum acceptable connection interval (20 ms), Connection interval uses 1.25 ms units. */
#define MAX_CONN_INTERVAL               MSEC_TO_UNITS(75, UNIT_1_25_MS)             /**< Maximum acceptable connection interval (75 ms), Connection interval uses 1.25 ms units. */
#define SLAVE_LATENCY                   0                                           /**< Slave latency. */
#define CONN_SUP_TIMEOUT                MSEC_TO_UNITS(4000, UNIT_10_MS)             /**< Connection supervisory timeout (4 seconds), Supervision Timeout uses 10 ms units. */
#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(5000)                       /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000)                      /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                           /**< Number of attempts before giving up the connection parameter negotiation. */

#define DEAD_BEEF                       0xDEADBEEF                                  /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

#define UART_TX_BUF_SIZE                256                                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE                256                                         /**< UART RX buffer size. */


BLE_NUS_DEF(m_nus, NRF_SDH_BLE_TOTAL_LINK_COUNT);                                   /**< BLE NUS service instance. */
NRF_BLE_GATT_DEF(m_gatt);                                                           /**< GATT module instance. */
NRF_BLE_QWR_DEF(m_qwr);                                                             /**< Context for the Queued Write module.*/
BLE_ADVERTISING_DEF(m_advertising);                                                 /**< Advertising module instance. */

static void advertising_start(bool erase_bonds);

uint8_t dust_data[6];

static uint16_t   m_conn_handle          = BLE_CONN_HANDLE_INVALID;                 /**< Handle of the current connection. */
static uint16_t   m_ble_nus_max_data_len = BLE_GATT_ATT_MTU_DEFAULT - 3;            /**< Maximum length of data (in bytes) that can be transmitted to the peer by the Nordic UART service module. */
static ble_uuid_t m_adv_uuids[]          =                                          /**< Universally unique service identifier. */
{
    {BLE_UUID_NUS_SERVICE, NUS_SERVICE_UUID_TYPE}
};

/**@brief Function for handling Peer Manager events.
2. *
3. * @param[in] p_evt Peer Manager event.
4. */
static void pm_evt_handler(pm_evt_t const * p_evt)
{
   ret_code_t err_code;
   switch (p_evt->evt_id)
   {
      case PM_EVT_BONDED_PEER_CONNECTED:
			{
				NRF_LOG_INFO("Connected to a previously bonded device.");
			} break;
	
			case PM_EVT_CONN_SEC_SUCCEEDED:
			{
				NRF_LOG_INFO("Connection secured: role: %d, conn_handle: 0x%x, procedure: %d.",
				ble_conn_state_role(p_evt->conn_handle),
				p_evt->conn_handle,
	      p_evt->params.conn_sec_succeeded.procedure);
			} break;

			case PM_EVT_CONN_SEC_FAILED:
			{
      /* Often, when securing fails, it shouldn't be restarted, for security reasons.
			 * Other times, it can be restarted directly.
			 * Sometimes it can be restarted, but only after changing some Security Parameters.
			 * Sometimes, it cannot be restarted until the link is disconnected and reconnected.
			 * Sometimes it is impossible, to secure the link, or the peer device does not support it.
			 * How to handle this error is highly application dependent. */
			} break;
 
			case PM_EVT_CONN_SEC_CONFIG_REQ:
			{
				// Reject pairing request from an already bonded peer.
				pm_conn_sec_config_t conn_sec_config = {.allow_repairing = false};
				pm_conn_sec_config_reply(p_evt->conn_handle, &conn_sec_config);
			} break;
 
			case PM_EVT_STORAGE_FULL:
			{
			// Run garbage collection on the flash.
				err_code = fds_gc();
				if (err_code == FDS_ERR_NO_SPACE_IN_QUEUES)
				{
				// Retry.
				}
				else
				{
					APP_ERROR_CHECK(err_code);
				}
			} break;
 
			case PM_EVT_PEERS_DELETE_SUCCEEDED:
			{
				advertising_start(false);
			} break;
			
			case PM_EVT_PEER_DATA_UPDATE_FAILED:
			{
				//Assert.
				APP_ERROR_CHECK(p_evt->params.peer_data_update_failed.error);
			} break;
			case PM_EVT_PEER_DELETE_FAILED:
			{
				//Assert.
				APP_ERROR_CHECK(p_evt->params.peer_delete_failed.error);
			} break;
 
			case PM_EVT_PEERS_DELETE_FAILED:
			{
				//Assert.
				APP_ERROR_CHECK(p_evt->params.peers_delete_failed_evt.error);
			} break;
 
			case PM_EVT_ERROR_UNEXPECTED:
      {
				//Assert.
				APP_ERROR_CHECK(p_evt->params.error_unexpected.error);
			} break;
 
      case PM_EVT_CONN_SEC_START:
			case PM_EVT_PEER_DATA_UPDATE_SUCCEEDED:
			case PM_EVT_PEER_DELETE_SUCCEEDED:
			case PM_EVT_LOCAL_DB_CACHE_APPLIED:
			case PM_EVT_LOCAL_DB_CACHE_APPLY_FAILED:
			// This can happen when the local DB has changed.
			case PM_EVT_SERVICE_CHANGED_IND_SENT:
	    case PM_EVT_SERVICE_CHANGED_IND_CONFIRMED:
			default:
			break;
		}
}

static void peer_manager_init()
{
		ble_gap_sec_params_t sec_param;
		ret_code_t err_code;
 
		err_code = pm_init();
		APP_ERROR_CHECK(err_code);
 
		memset(&sec_param, 0, sizeof(ble_gap_sec_params_t));
	  // Security parameters to be used for all security procedures.
	  sec_param.bond = SEC_PARAM_BOND;
    sec_param.mitm = SEC_PARAM_MITM;
		sec_param.lesc = SEC_PARAM_LESC;
		sec_param.keypress = SEC_PARAM_KEYPRESS;
		sec_param.io_caps = SEC_PARAM_IO_CAPABILITIES;
		sec_param.oob = SEC_PARAM_OOB;
		sec_param.min_key_size = SEC_PARAM_MIN_KEY_SIZE;
		sec_param.max_key_size = SEC_PARAM_MAX_KEY_SIZE;
		sec_param.kdist_own.enc = 1;
		sec_param.kdist_own.id = 1;
		sec_param.kdist_peer.enc = 1;
		sec_param.kdist_peer.id = 1;
 
		err_code = pm_sec_params_set(&sec_param);
		APP_ERROR_CHECK(err_code);
 
		err_code = pm_register(pm_evt_handler);
		APP_ERROR_CHECK(err_code);
	}

/** @brief Clear bonding information from persistent storage.
*/
static void delete_bonds(void)
{
		ret_code_t err_code;
		NRF_LOG_INFO("Erase bonds!");
	
		err_code = pm_peers_delete();
		APP_ERROR_CHECK(err_code);
}

/**@brief Function for assert macro callback.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyse
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num    Line number of the failing ASSERT call.
 * @param[in] p_file_name File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}



static void timer1_timeout_handler(void)
{
	  static uint16_t Lux;
	  uint32_t err_code;
	  uint16_t length = 6;
	  static uint16_t PM25;
	  
	  PM25 = dust_data[0];
		PM25 <= 8;
		PM25 |= dust_data[1];
	
	  BH1750_Cmd();
	  LCD_P20x40Ch(34,1,PM25/100+'0');
		LCD_P20x40Ch(54,1,PM25/10%10+'0');
		LCD_P20x40Ch(74,1,PM25%10+'0');
		LCD_P6x8Str(70,6," ug/m3");
		nrf_delay_ms(15);
	  Lux = BH1750_Get();
		Draw_line(2, 0,Lux/4+10);
	  LCD_setContrast(Lux/4);
	 
	  do
    {
       err_code = ble_nus_data_send(&m_nus, dust_data, &length, m_conn_handle);
       if ( (err_code != NRF_ERROR_INVALID_STATE) && (err_code != NRF_ERROR_BUSY) &&
            (err_code != NRF_ERROR_NOT_FOUND) )
       {
          APP_ERROR_CHECK(err_code);
       }
    } while (err_code == NRF_ERROR_BUSY);
								
	  while(app_uart_put(0x42) != NRF_SUCCESS);
		while(app_uart_put(0x4d) != NRF_SUCCESS);
		while(app_uart_put(0xe2) != NRF_SUCCESS);
		while(app_uart_put(0x00) != NRF_SUCCESS);
		while(app_uart_put(0x00) != NRF_SUCCESS);
		while(app_uart_put(0x01) != NRF_SUCCESS);
		while(app_uart_put(0x71) != NRF_SUCCESS);
		
}

APP_TIMER_DEF(m_timer_id1);  
#define TIME_LEVEL_MEAS_INTERVAL          APP_TIMER_TICKS(5000)
/**@brief Function for initializing the timer module.
 */
static void timers_init(void)//ʱʼ
{
    ret_code_t err_code = app_timer_init();
    APP_ERROR_CHECK(err_code);
	
	  err_code = app_timer_create(&m_timer_id1,APP_TIMER_MODE_REPEATED,timer1_timeout_handler);
	  APP_ERROR_CHECK(err_code);
}

/**@brief Function for the GAP initialization.
 *
 * @details This function will set up all the necessary GAP (Generic Access Profile) parameters of
 *          the device. It also sets the permissions and appearance.
 */
static void gap_params_init(void)//GAPʼ
{
    uint32_t                err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);

    err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *) DEVICE_NAME,
                                          strlen(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling Queued Write Module errors.
 *
 * @details A pointer to this function will be passed to each service which may need to inform the
 *          application about an error.
 *
 * @param[in]   nrf_error   Error code containing information about what went wrong.
 */
static void nrf_qwr_error_handler(uint32_t nrf_error)//д
{
    APP_ERROR_HANDLER(nrf_error);
}


//static uint8_t data_array[BLE_NUS_MAX_DATA_LEN];
static uint8_t states[4];
//static uint8_t index = 0;
static uint8_t change_state=0;


/**@brief Function for handling the data from the Nordic UART Service.
 *
 * @details This function will process the data received from the Nordic UART BLE Service and send
 *          it to the UART module.
 *
 * @param[in] p_evt       Nordic UART Service event.
 */
/**@snippet [Handling the data received over BLE] */
static void nus_data_handler(ble_nus_evt_t * p_evt)//жϲ
{

    if (p_evt->type == BLE_NUS_EVT_RX_DATA)
    {

        NRF_LOG_DEBUG("Received data from BLE NUS. Writing data on UART.");
        NRF_LOG_HEXDUMP_DEBUG(p_evt->params.rx_data.p_data, p_evt->params.rx_data.length);
			
			 if((p_evt->params.rx_data.p_data[0]=='A')&&(p_evt->params.rx_data.p_data[1]=='T')&&(p_evt->params.rx_data.p_data[2]=='+'))
		{
			
	     if(((p_evt->params.rx_data.p_data[3]=='O')||(p_evt->params.rx_data.p_data[3]=='o'))&&((p_evt->params.rx_data.p_data[4]=='N')||(p_evt->params.rx_data.p_data[4]=='n')))  //յΪON"on"
		{
		 nrf_gpio_pin_set(3 ); //ӷ	
		 LCD_P6x8Str(59,7,"ON ");
		 states[0]=p_evt->params.rx_data.p_data[0];               //Ҫϴ״̬
		 states[1]=p_evt->params.rx_data.p_data[1];
		 states[2]=0;
		 change_state=1;                  //״̬仯ϴ״̬һ
		}
		
		 if(((p_evt->params.rx_data.p_data[3]=='O')||(p_evt->params.rx_data.p_data[3]=='o'))&&((p_evt->params.rx_data.p_data[4]=='F')||(p_evt->params.rx_data.p_data[4]=='f'))&&((p_evt->params.rx_data.p_data[5]=='F')||(p_evt->params.rx_data.p_data[5]=='f')))
		{
		 //յΪOFF"off"
		 nrf_gpio_pin_clear(3 ); //ӷֹͣ
		 LCD_P6x8Str(59,7,"OFF");	
		 states[0]=p_evt->params.rx_data.p_data[0];             //Ҫϴ״̬
		 states[1]=p_evt->params.rx_data.p_data[1];
		 states[2]=p_evt->params.rx_data.p_data[2];
		 change_state=1;                  //״̬仯ϴ״̬һ
		}
		
		if(((p_evt->params.rx_data.p_data[3]>='0')&&(p_evt->params.rx_data.p_data[3]<='9'))&&((p_evt->params.rx_data.p_data[4]>='0')||(p_evt->params.rx_data.p_data[4]<='9')))  //յΪλ
		{
		 set_motor_speed((uint16_t)((p_evt->params.rx_data.p_data[3]-'0')*10+p_evt->params.rx_data.p_data[4]-'0'));
		 motor_speed_disp();
		 states[0]=p_evt->params.rx_data.p_data[0];               //Ҫϴ״̬
		 states[1]=p_evt->params.rx_data.p_data[1];
		 states[2]=0;
		 change_state=1;                  //״̬仯ϴ״̬һ
		}
	}
        if (p_evt->params.rx_data.p_data[p_evt->params.rx_data.length - 1] == '\r')
        {
            while (app_uart_put('\n') == NRF_ERROR_BUSY);
        }
    
	}

}

//dfu
static void ble_dfu_evt_handler(ble_dfu_buttonless_evt_type_t event)
{
		switch (event)
		{
			case BLE_DFU_EVT_BOOTLOADER_ENTER_PREPARE:
			NRF_LOG_INFO("Device is preparing to enter bootloader mode.");
			// YOUR_JOB: Disconnect all bonded devices that currently are connected.
			// This is required to receive a service changed indication
			// on bootup after a successful (or aborted) Device Firmware Update.
			fill_picture(0x00);
			LCD_P8x16Str(32,3,"DFU Mode");
			break;
 
			case BLE_DFU_EVT_BOOTLOADER_ENTER:
			// YOUR_JOB: Write app-specific unwritten data to FLASH, control finalization of this
			// by delaying reset by reporting false in app_shutdown_handler
			NRF_LOG_INFO("Device will enter bootloader mode.");
			break;
		
			case BLE_DFU_EVT_BOOTLOADER_ENTER_FAILED:
			NRF_LOG_ERROR("Request to enter bootloader mode failed asynchroneously.");
			// YOUR_JOB: Take corrective measures to resolve the issue
			// like calling APP_ERROR_CHECK to reset the device.
			break;
		
			case BLE_DFU_EVT_RESPONSE_SEND_ERROR:
			NRF_LOG_ERROR("Request to send a response to client failed.");
			// YOUR_JOB: Take corrective measures to resolve the issue
			// like calling APP_ERROR_CHECK to reset the device.
			APP_ERROR_CHECK(false);
			break;
	
			default:
			NRF_LOG_ERROR("Unknown event from ble_dfu_buttonless.");
			break;
		}
}

 /**@brief Handler for shutdown preparation.
*
* @details During shutdown procedures, this function will be called at a 1 second interval
* untill the function returns true. When the function returns true, it means that the
* app is ready to reset to DFU mode.
* @param[in] event Power manager event.
* @retval True if shutdown is allowed by this power manager handler, otherwise false.
*/
static bool app_shutdown_handler(nrf_pwr_mgmt_evt_t event)
{
		switch (event)
		{
			case NRF_PWR_MGMT_EVT_PREPARE_DFU:
			NRF_LOG_INFO("Power management wants to reset to DFU mode.");
			// YOUR_JOB: Get ready to reset into DFU mode
			//
			// If you aren't finished with any ongoing tasks, return "false" to
			// signal to the system that reset is impossible at this stage.
			//
			// Here is an example using a variable to delay resetting the device.
			//
			// if (!m_ready_for_reset)
			// {
			// return false;
			// }
			// else
			//{
			//
			// // Device ready to enter
			// uint32_t err_code;
			// err_code = sd_softdevice_disable();
			// APP_ERROR_CHECK(err_code);
			// err_code = app_timer_stop_all();
			// APP_ERROR_CHECK(err_code);
			//}
			
			break;
 
			default:
			// YOUR_JOB: Implement any of the other events available from the power management module:
			// -NRF_PWR_MGMT_EVT_PREPARE_SYSOFF
			// -NRF_PWR_MGMT_EVT_PREPARE_WAKEUP
     // -NRF_PWR_MGMT_EVT_PREPARE_RESET
			return true;
		}
 
    NRF_LOG_INFO("Power management allowed to reset to DFU mode.");
    return true;
}

//lint -esym(528, m_app_shutdown_handler)
/**@brief Register application shutdown handler with priority 0.
*/
NRF_PWR_MGMT_HANDLER_REGISTER(app_shutdown_handler, 0);

/**@snippet [Handling the data received over BLE] */

/**@snippet [Handling the data received over BLE] */


/**@brief Function for initializing services that will be used by the application.
 */
static void services_init(void)//ʼ
{
    uint32_t           err_code;
    ble_nus_init_t     nus_init;
    nrf_ble_qwr_init_t qwr_init = {0};
    ble_dfu_buttonless_init_t dfus_init = {0};
    // Initialize Queued Write Module.
    qwr_init.error_handler = nrf_qwr_error_handler;

    err_code = nrf_ble_qwr_init(&m_qwr, &qwr_init);
    APP_ERROR_CHECK(err_code);

    // Initialize NUS.
    memset(&nus_init, 0, sizeof(nus_init));

    nus_init.data_handler = nus_data_handler;

    err_code = ble_nus_init(&m_nus, &nus_init);
    APP_ERROR_CHECK(err_code);
		
		// Initialize the async SVCI interface to bootloader.
		err_code = ble_dfu_buttonless_async_svci_init();
		APP_ERROR_CHECK(err_code);
 
		dfus_init.evt_handler = ble_dfu_evt_handler;
 
		err_code = ble_dfu_buttonless_init(&dfus_init);
		APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling an event from the Connection Parameters Module.
 *
 * @details This function will be called for all events in the Connection Parameters Module
 *          which are passed to the application.
 *
 * @note All this function does is to disconnect. This could have been done by simply setting
 *       the disconnect_on_fail config parameter, but instead we use the event handler
 *       mechanism to demonstrate its use.
 *
 * @param[in] p_evt  Event received from the Connection Parameters Module.
 */
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)//Ӳ¼
{
    uint32_t err_code;

    if (p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED)
    {
        err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
        APP_ERROR_CHECK(err_code);
    }
}


/**@brief Function for handling errors from the Connection Parameters module.
 *
 * @param[in] nrf_error  Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error)//Ӳ
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for initializing the Connection Parameters module.
 */
static void conn_params_init(void)//Ӳʼ
{
    uint32_t               err_code;
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;

    err_code = ble_conn_params_init(&cp_init);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for putting the chip into sleep mode.
 *
 * @note This function will not return.
 */
static void sleep_mode_enter(void)//ģʽ
{
    uint32_t err_code = bsp_indication_set(BSP_INDICATE_IDLE);
    APP_ERROR_CHECK(err_code);

    // Prepare wakeup buttons.
    err_code = bsp_btn_ble_sleep_mode_prepare();
    APP_ERROR_CHECK(err_code);

    // Go to system-off mode (this function will not return; wakeup will cause a reset).
    err_code = sd_power_system_off();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling advertising events.
 *
 * @details This function will be called for advertising events which are passed to the application.
 *
 * @param[in] ble_adv_evt  Advertising event.
 */
static void on_adv_evt(ble_adv_evt_t ble_adv_evt)//㲥¼
{
    uint32_t err_code;

    switch (ble_adv_evt)
    {
        case BLE_ADV_EVT_FAST:
            err_code = bsp_indication_set(BSP_INDICATE_ADVERTISING);
            APP_ERROR_CHECK(err_code);
            break;
        case BLE_ADV_EVT_IDLE:
            sleep_mode_enter();
            break;
        default:
            break;
    }
}


/**@brief Function for handling BLE events.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 * @param[in]   p_context   Unused.
 */
static void ble_evt_handler(ble_evt_t const * p_ble_evt, void * p_context)//¼
{
    uint32_t err_code;

    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            NRF_LOG_INFO("Connected");
            err_code = bsp_indication_set(BSP_INDICATE_CONNECTED);
            APP_ERROR_CHECK(err_code);
            m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
            err_code = nrf_ble_qwr_conn_handle_assign(&m_qwr, m_conn_handle);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GAP_EVT_DISCONNECTED:
            NRF_LOG_INFO("Disconnected");
            // LED indication will be changed when advertising starts.
            m_conn_handle = BLE_CONN_HANDLE_INVALID;
            break;

        case BLE_GAP_EVT_PHY_UPDATE_REQUEST:
        {
            NRF_LOG_DEBUG("PHY update request.");
            ble_gap_phys_t const phys =
            {
                .rx_phys = BLE_GAP_PHY_AUTO,
                .tx_phys = BLE_GAP_PHY_AUTO,
            };
            err_code = sd_ble_gap_phy_update(p_ble_evt->evt.gap_evt.conn_handle, &phys);
            APP_ERROR_CHECK(err_code);
        } break;

        case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
            // Pairing not supported
            err_code = sd_ble_gap_sec_params_reply(m_conn_handle, BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP, NULL, NULL);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_SYS_ATTR_MISSING:
            // No system attributes have been stored.
            err_code = sd_ble_gatts_sys_attr_set(m_conn_handle, NULL, 0, 0);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTC_EVT_TIMEOUT:
            // Disconnect on GATT Client timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gattc_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;

        case BLE_GATTS_EVT_TIMEOUT:
            // Disconnect on GATT Server timeout event.
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gatts_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            APP_ERROR_CHECK(err_code);
            break;

        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for the SoftDevice initialization.
 *
 * @details This function initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)//Эջʼ
{
    ret_code_t err_code;

    err_code = nrf_sdh_enable_request();
    APP_ERROR_CHECK(err_code);

    // Configure the BLE stack using the default settings.
    // Fetch the start address of the application RAM.
    uint32_t ram_start = 0;
    err_code = nrf_sdh_ble_default_cfg_set(APP_BLE_CONN_CFG_TAG, &ram_start);
    APP_ERROR_CHECK(err_code);

    // Enable BLE stack.
    err_code = nrf_sdh_ble_enable(&ram_start);
    APP_ERROR_CHECK(err_code);

    // Register a handler for BLE events.
    NRF_SDH_BLE_OBSERVER(m_ble_observer, APP_BLE_OBSERVER_PRIO, ble_evt_handler, NULL);
}


/**@brief Function for handling events from the GATT library. */
void gatt_evt_handler(nrf_ble_gatt_t * p_gatt, nrf_ble_gatt_evt_t const * p_evt)//GATT¼
{
    if ((m_conn_handle == p_evt->conn_handle) && (p_evt->evt_id == NRF_BLE_GATT_EVT_ATT_MTU_UPDATED))
    {
        m_ble_nus_max_data_len = p_evt->params.att_mtu_effective - OPCODE_LENGTH - HANDLE_LENGTH;
        NRF_LOG_INFO("Data len is set to 0x%X(%d)", m_ble_nus_max_data_len, m_ble_nus_max_data_len);
    }
    NRF_LOG_DEBUG("ATT MTU exchange completed. central 0x%x peripheral 0x%x",
                  p_gatt->att_mtu_desired_central,
                  p_gatt->att_mtu_desired_periph);
}


/**@brief Function for initializing the GATT library. */
void gatt_init(void)//GATTʼ
{
    ret_code_t err_code;

    err_code = nrf_ble_gatt_init(&m_gatt, gatt_evt_handler);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_ble_gatt_att_mtu_periph_set(&m_gatt, NRF_SDH_BLE_GATT_MAX_MTU_SIZE);
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling events from the BSP module.
 *
 * @param[in]   event   Event generated by button press.
 */
void bsp_event_handler(bsp_event_t event)//弶¼
{
    uint32_t err_code;
    switch (event)
    {
        case BSP_EVENT_SLEEP:
            sleep_mode_enter();
            break;

        case BSP_EVENT_DISCONNECT:
            err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
            if (err_code != NRF_ERROR_INVALID_STATE)
            {
                APP_ERROR_CHECK(err_code);
            }
            break;

        case BSP_EVENT_WHITELIST_OFF:
            if (m_conn_handle == BLE_CONN_HANDLE_INVALID)
            {
                err_code = ble_advertising_restart_without_whitelist(&m_advertising);
                if (err_code != NRF_ERROR_INVALID_STATE)
                {
                    APP_ERROR_CHECK(err_code);
                }
            }
            break;

        default:
            break;
    }
}


/**@brief   Function for handling app_uart events.
 *
 * @details This function will receive a single character from the app_uart module and append it to
 *          a string. The string will be be sent over BLE when the last character received was a
 *          'new line' '\n' (hex 0x0A) or if the string has reached the maximum data length.
 */
/**@snippet [Handling the data received over UART] */
void uart_event_handle(app_uart_evt_t * p_event)
{
    static uint8_t data_array[35];
    static uint8_t index = 0;
    switch (p_event->evt_type)
    {
        case APP_UART_DATA_READY:
            UNUSED_VARIABLE(app_uart_get(&data_array[index]));
            index++;

            if (index >= 32)
            {
							if((data_array[0] == 0x42) && (data_array[1] == 0x4d))
							{
                //NRF_LOG_DEBUG("Ready to send data over BLE NUS");
                //NRF_LOG_HEXDUMP_DEBUG(data_array, index);
							  nrf_gpio_pin_toggle(28);
								//pm25
								dust_data[1] = data_array[13];
							  dust_data[0] = data_array[12];
								//pm1.0
								dust_data[3] = data_array[11];
							  dust_data[2] = data_array[10];
								//pm10
								dust_data[5] = data_array[15];
							  dust_data[4] = data_array[14];
								
						  }
							
							index = 0;
            }
            break;

        case APP_UART_COMMUNICATION_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;

        default:
            break;
    }
}
/**@snippet [Handling the data received over UART] */


/**@brief  Function for initializing the UART module.
 */
/**@snippet [UART Initialization] */
static void uart_init(void)
{
    uint32_t                     err_code;
    app_uart_comm_params_t const comm_params =
    {
        .rx_pin_no    = 12,
        .tx_pin_no    = 11,
        //.rts_pin_no   = RTS_PIN_NUMBER,
        //.cts_pin_no   = CTS_PIN_NUMBER,
        .flow_control = APP_UART_FLOW_CONTROL_DISABLED,
        .use_parity   = false,
        .baud_rate    = NRF_UART_BAUDRATE_9600
    };

    APP_UART_FIFO_INIT(&comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_event_handle,
                       APP_IRQ_PRIORITY_LOWEST,
                       err_code);
    APP_ERROR_CHECK(err_code);
}
/**@snippet [UART Initialization] */

#define TX_POWER_LEVEL (-20)
/**@brief Function for initializing the Advertising functionality.
 */
static void advertising_init(void)
{
    uint32_t               err_code;
    ble_advertising_init_t init;
    int8_t tx_power_level = TX_POWER_LEVEL;
	
    memset(&init, 0, sizeof(init));

    init.advdata.name_type          = BLE_ADVDATA_FULL_NAME;
    init.advdata.include_appearance = false;
    init.advdata.flags              = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;
    init.advdata.p_tx_power_level = &tx_power_level;
	
    init.srdata.uuids_complete.uuid_cnt = sizeof(m_adv_uuids) / sizeof(m_adv_uuids[0]);
    init.srdata.uuids_complete.p_uuids  = m_adv_uuids;

    init.config.ble_adv_fast_enabled  = true;
    init.config.ble_adv_fast_interval = APP_ADV_INTERVAL;
    init.config.ble_adv_fast_timeout  = APP_ADV_DURATION;
    init.evt_handler = on_adv_evt;

    err_code = ble_advertising_init(&m_advertising, &init);
    APP_ERROR_CHECK(err_code);

    ble_advertising_conn_cfg_tag_set(&m_advertising, APP_BLE_CONN_CFG_TAG);
}

/**@brief Function for initializing the nrf log module.
 */
static void log_init(void)
{
    ret_code_t err_code = NRF_LOG_INIT(NULL);
    APP_ERROR_CHECK(err_code);

    NRF_LOG_DEFAULT_BACKENDS_INIT();
}


/**@brief Function for initializing power management.
 */
static void power_management_init(void)
{
    ret_code_t err_code;
    err_code = nrf_pwr_mgmt_init();
    APP_ERROR_CHECK(err_code);
}


/**@brief Function for handling the idle state (main loop).
 *
 * @details If there is no pending log operation, then sleep until next the next event occurs.
 */
static void idle_state_handle(void)
{
    UNUSED_RETURN_VALUE(NRF_LOG_PROCESS());
    nrf_pwr_mgmt_run();
}


/**@brief Function for starting advertising.
 */
static void advertising_start(bool erase_bonds)
{
	  if (erase_bonds == true)
		{
			delete_bonds();
		}
		else
		{
			uint32_t err_code = ble_advertising_start(&m_advertising, BLE_ADV_MODE_FAST);
			APP_ERROR_CHECK(err_code);
		}
}

static void application_timers_start(void)
{
   uint32_t err_code;

   err_code = app_timer_start(m_timer_id1, TIME_LEVEL_MEAS_INTERVAL, NULL);
   APP_ERROR_CHECK(err_code);
}

unsigned char lux[] = {
	0x80,0x80,0x0C,0x0C,0xC0,0xE0,0xF0,0xF3,0xF3,0xF0,0xE0,0xC0,0x0C,0x0C,0x80,0x80,
0x01,0x01,0x30,0x30,0x03,0x07,0x0F,0xCF,0xCF,0x0F,0x07,0x03,0x30,0x30,0x01,0x01,
};
unsigned char fan[] = {
  0x00,0x81,0x43,0x45,0x49,0xD1,0x61,0x21,0x22,0x7C,0xC0,0x20,0x10,0x08,0x04,0xFE,
0x7F,0x20,0x10,0x08,0x04,0x03,0x3E,0x44,0x84,0x86,0x8B,0x92,0xA2,0xC2,0x81,0x00,
};
unsigned char gen[] = {
  0x38,0x44,0x92,0x91,0x91,0x92,0x44,0x38
};
/**@brief Application main function.
 */
int main(void)
{
	  bool erase_bonds;
	  
	  //enable dcdc converter
	  NRF_POWER->DCDCEN = 1;
	  //disable fpu
		__set_FPSCR(__get_FPSCR() & ~(0x0000009F));
		(void) __get_FPSCR();
		NVIC_ClearPendingIRQ(FPU_IRQn);
	
    // Initialize.
	  nrf_gpio_cfg_output(28);
	  nrf_gpio_cfg_output(3);
	  nrf_gpio_pin_set(3);
	  init_motor();
	  //init_dustsensor();

    // Enter main loop.
		//init PMS9003
    //log_init();
    timers_init();
    power_management_init();
    ble_stack_init();
	  peer_manager_init();
    gap_params_init();
    gatt_init();
    services_init();
    advertising_init();
    conn_params_init();
    advertising_start(erase_bonds);
    application_timers_start();
		
		LCD_Init();
		fill_picture(0x00);
		Draw_BMP(2,6,18,8,lux);
		Draw_BMP(112,6,128,8,fan);
		Draw_BMP(50,7,58,8,gen);
		LCD_P6x8Str(59,7,"ON ");
		motor_speed_disp();
		
		uart_init();
		
		while(app_uart_put(0x42) != NRF_SUCCESS);
		while(app_uart_put(0x4d) != NRF_SUCCESS);
		while(app_uart_put(0xe1) != NRF_SUCCESS);
		while(app_uart_put(0x00) != NRF_SUCCESS);
		while(app_uart_put(0x00) != NRF_SUCCESS);
		while(app_uart_put(0x01) != NRF_SUCCESS);
		while(app_uart_put(0x70) != NRF_SUCCESS);
		
	  for (;;)
    {
        idle_state_handle();
    }
}


/**
 * @}
 */
